'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _createChannel = require('./create-channel');

var _createChannel2 = _interopRequireDefault(_createChannel);

var _getChannelName = require('./get-channel-name');

var _getChannelName2 = _interopRequireDefault(_getChannelName);

var _subscriberConsumerFactory = require('./subscriber-consumer-factory');

var _subscriberConsumerFactory2 = _interopRequireDefault(_subscriberConsumerFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultExchangeOptions = {
  durable: false,
  autoDelete: false
};

const defaultQueueOptions = {
  exclusive: true,
  autoDelete: true
};

const defaultSubscriberOptions = {
  noAck: true,
  exclusive: true
};

const addSubscriber = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* (args, plugin) {
    try {
      const { connection, exchange, topic, subscriber, options } = args;
      const { state: pluginState } = plugin;
      const { _openChannels } = pluginState;

      const {
        channelName: userChannelName,
        channelOptions,
        exchangeOptions: userExchangeOptions,
        queueOptions: userQueueOptions,
        subscriberOptions: userSubscriberOptions
      } = options || {};

      const channelName = userChannelName || (0, _getChannelName2.default)({ method: 'subscribeToMessages', exchange });
      const exchangeTopic = topic ? `${exchange}.${topic}` : `${exchange}.*`;

      const exchangeOptions = (0, _defaultsDeep2.default)({}, userExchangeOptions, defaultExchangeOptions);
      const queueOptions = (0, _defaultsDeep2.default)({}, userQueueOptions, defaultQueueOptions);
      const subscriberOptions = (0, _defaultsDeep2.default)({}, userSubscriberOptions, defaultSubscriberOptions);

      let activeChannel;

      if (_openChannels[channelName]) {
        activeChannel = _openChannels[channelName].channel;
      } else {
        activeChannel = yield (0, _createChannel2.default)({
          name: channelName,
          options: channelOptions,
          persist: true,
          connection
        }, plugin);
      }

      yield activeChannel.assertExchange(exchange, 'topic', exchangeOptions);

      const activeQueue = yield activeChannel.assertQueue('', queueOptions);

      yield activeChannel.bindQueue(activeQueue.queue, exchange, exchangeTopic);

      const consumer = (0, _subscriberConsumerFactory2.default)({
        subscriber,
        channel: activeChannel,
        queue: activeQueue,
        options: subscriberOptions
      });

      const consumed = yield activeChannel.consume(activeQueue.queue, consumer, subscriberOptions);

      return {
        channel: activeChannel,
        queue: activeQueue,
        consumed
      };
    } catch (error) {
      throw error;
    }
  });

  return function addSubscriber(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

exports.default = addSubscriber;