'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _createChannel = require('./create-channel');

var _createChannel2 = _interopRequireDefault(_createChannel);

var _getChannelName = require('./get-channel-name');

var _getChannelName2 = _interopRequireDefault(_getChannelName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultQueueOptions = {
  durable: true,
  maxPriority: 0
};

const pushTask = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* (args, plugin) {
    const { connection, queue, payload, type, correlationId, options } = args;
    const { options: pluginOptions, state: pluginState } = plugin;
    const { preserveChannels, retry: retryOptions } = pluginOptions;
    const { _openChannels } = pluginState;

    const {
      channelName: userChannelName,
      channelOptions,
      queueOptions,
      taskOptions
    } = options || {};

    const channelName = userChannelName || (0, _getChannelName2.default)({ method: 'pushTask', queue });

    const defaultTaskOptions = {
      persistent: true,
      priority: 0,
      contentType: 'application/json',
      contentEncoding: 'utf-8',
      headers: {
        'x-retry-count': retryOptions === false ? -1 : 0
      }
    };

    const mergedTaskOptions = (0, _defaultsDeep2.default)({}, taskOptions, defaultTaskOptions);

    if (type) {
      mergedTaskOptions.type = type;
    }

    if (correlationId) {
      mergedTaskOptions.correlationId = String(correlationId);
    }

    let activeChannel;

    if (_openChannels[channelName]) {
      activeChannel = _openChannels[channelName].channel;
    } else {
      activeChannel = yield (0, _createChannel2.default)({
        name: channelName,
        options: channelOptions,
        connection
      }, plugin);
    }

    const activeQueue = yield activeChannel.assertQueue(queue, (0, _defaultsDeep2.default)({}, queueOptions, defaultQueueOptions));

    const queued = yield activeChannel.sendToQueue(activeQueue.queue, new Buffer((0, _stringify2.default)(payload), mergedTaskOptions.contentEncoding), mergedTaskOptions);

    if (!preserveChannels) {
      yield activeChannel.close();
    }

    return {
      channel: activeChannel,
      queue: activeQueue,
      queued
    };
  });

  return function pushTask(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

exports.default = pushTask;